#include "interface.h"

using namespace std;

/*
 * Global Variables:
 * mice:         vector of `Mouse`, indexed by their id
 * orders:       vector of `Order`, the orders you should fulfill
 * my_id:        zero-based index of your Mouse in the Mice list.
 * storages:     Struct containing information about the table, bowls and muffin cups
 * contexts:     Struct containing information about the different stations
 * current_tick: Current tick
 */

void mouse_going_to_dough_station(const Mouse& mouse, uint32_t arrival_tick) {
    cerr << "Mouse " << mouse.id << " is going to arrive at the dough station on tick " << arrival_tick << "\n";
}

void mouse_going_to_chocolate_station(const Mouse& mouse, uint32_t arrival_tick) {
    cerr << "Mouse " << mouse.id << " is going to arrive at the chocolate station on tick " << arrival_tick << "\n";
}

void mouse_going_to_lemon_station(const Mouse& mouse, uint32_t arrival_tick) {
    cerr << "Mouse " << mouse.id << " is going to arrive at the lemons station on tick " << arrival_tick << "\n";
}

void mouse_going_to_bowl_filling_station(const Mouse& mouse, uint32_t arrival_tick) {
    cerr << "Mouse " << mouse.id << " is going to arrive at the bowl filling station on tick " << arrival_tick << "\n";
}

void mouse_going_to_mixing_station(const Mouse& mouse, uint32_t arrival_tick) {
    cerr << "Mouse " << mouse.id << " is going to arrive at the mixing station on tick " << arrival_tick << "\n";
}

void mouse_going_to_cup_filling_station(const Mouse& mouse, uint32_t arrival_tick) {
    cerr << "Mouse " << mouse.id << " is going to arrive at the cup filling station on tick " << arrival_tick << "\n";
}

void mouse_going_to_baking_station(const Mouse& mouse, uint32_t arrival_tick) {
    cerr << "Mouse " << mouse.id << " is going to arrive at the baking station on tick " << arrival_tick << "\n";
}

void mouse_getting_dough(const Mouse& mouse, uint32_t finish_tick) {
    cerr << "Mouse " << mouse.id << " is getting " << contexts.dough.max_amount << " dough and will finish on tick " << finish_tick << "\n";
}
        
void mouse_getting_chocolate(const Mouse& mouse, uint32_t finish_tick) {
    cerr << "Mouse " << mouse.id << " is getting " << contexts.chocolate.max_amount << " chocolate and will finish on tick " << finish_tick << "\n";
}

void mouse_getting_lemons(const Mouse& mouse, uint32_t finish_tick) {
    cerr << "Mouse " << mouse.id << " is getting " << contexts.lemons.max_amount << " lemons and will finish on tick " << finish_tick << "\n";
}

void mouse_filling_bowls(const Mouse& mouse, Bowl bowl, uint32_t resource_id, const Storages& old_storages, uint32_t finish_tick) {
    if (resource_id == 1) {
        cerr << "Mouse " << mouse.id << " is filling 1 chocolates and 1 dough into bowl " << bowl.id << " and will finish on tick " << finish_tick << "\n";
    } else if (resource_id == 2) {
        cerr << "Mouse " << mouse.id << " is filling 1 lemons and 1 dough into bowl " << bowl.id << " and will finish on tick " << finish_tick << "\n";
    }
}

void mouse_mixing_bowl(const Mouse& mouse, Bowl bowl, uint32_t finish_tick) {
    cerr << "Mouse " << mouse.id << " is mixing the contents of bowl " << bowl.id << " and will finish on tick " << finish_tick << "\n";
}

void mouse_filling_cups(const Mouse& mouse, Bowl bowl, const Storages& old_storages, uint32_t finish_tick) {
    cerr << "Mouse " << mouse.id << " is filling 1 cups from the contents of bowl " << bowl.id << " and will finish on tick " << finish_tick << "\n";
}

uint32_t baking_until = 0;
void mouse_baking_muffins(const Mouse& mouse, const Storages& old_storages, uint32_t finish_tick) {
    cerr << "Mouse " << mouse.id << " is putting the " << old_storages.cups[1] + old_storages.cups[2] << " cups into the oven and will finish on tick " << finish_tick << ". The oven will finish baking on tick " << finish_tick + contexts.baking.baking_time << "\n";
    baking_until = finish_tick + contexts.baking.baking_time;
}

void mouse_idling(const Mouse& mouse, uint32_t finish_tick) {
    cerr << "Mouse " << mouse.id << " is AFKing until " << finish_tick << ", what a noob.\n";
}

void print_mouse_stats() {
        
    cerr << "My mouse has id " << my_id << "\n";
    cerr << "My mouse takes " << mice[my_id].dough_fetching_speed << " ticks to fetch dough\n";
    cerr << "My mouse takes " << mice[my_id].chocolate_fetching_speed << " ticks to fetch chocolate\n";
    cerr << "My mouse takes " << mice[my_id].lemon_fetching_speed << " ticks to fetch lemons\n";
    cerr << "My mouse takes " << mice[my_id].bowl_filling_speed << " ticks per resource to fill a bowl\n";
    cerr << "My mouse takes " << mice[my_id].mixing_speed << " ticks to mix a bowl\n";
    cerr << "My mouse takes " << mice[my_id].cup_filling_speed << " ticks per resource to fill into cups\n";
    cerr << "My mouse takes " << mice[my_id].baking_speed << " ticks to move all cups into the oven\n";
}

void print_orders() {

    cerr << "There are " << orders.size() << " orders:\n";
    for (const Order& order : orders) {
        if (order.muffin_type == 1) {
            cerr << order.amount << " chocolate muffins should be finished by tick " << order.tick << "\n";
        }else if (order.muffin_type == 2) {
            cerr << order.amount << " lemon muffins should be finished by tick " << order.tick << "\n";
        }
    }
}

void print_storages() {

    cerr << "There are " << storages.bowls.size() << " bowls with a capacity of " << storages.bowls[0].capacity << " available\n";
    cerr << "There are " << storages.max_cups << " muffin cups available\n";
}

void make_muffins() {
    print_mouse_stats();
    print_orders();
    print_storages();

    while (true) {    
        go_to(contexts.dough); // go to dough storage
        get_dough();
        go_to(contexts.chocolate); // go to chocolate storage
        get_chocolate();
        go_to(contexts.lemons); // go to lemon storage
        get_lemons();
        go_to(contexts.bowl_filling); // go to bowl filling station
        while(storages.bowls[0].in_use || storages.bowls[0].mixed) {
            idle();
        }
        fill_bowl(storages.bowls[0], 1); // fill bowl 0 with 1 unit of chocolate and one unit of dough
        go_to(contexts.mixing); // go to bowl mixing station
        if (!storages.bowls[0].mixed) {
            while(storages.bowls[0].in_use) {
                idle();
            }
            if (!storages.bowls[0].mixed){                    
                mix_bowl(storages.bowls[0]); // mix bowl 0
            }
        }
        go_to(contexts.cup_filling); // go to cup filling station
        while(storages.bowls[0].in_use) {
            idle();
        }
        fill_cups(storages.bowls[0]); //fill 1 cup with chocolate dough
        fill_cups(storages.bowls[0]); //fill another cup with chocolate dough
        go_to(contexts.baking); // go to oven
        if (baking_until > current_tick) {
            idle(baking_until);
        }
        bake_muffins(); //bake the 2 cups
    }            
}

